@@ -1,6 +1,6 @@
 from sympy.core.assumptions import StdFactKB, _assume_defined
 from sympy.core.compatibility import is_sequence, ordered
-from .basic import Basic
+from .basic import Basic, Atom
 from .sympify import sympify
 from .singleton import S
 from .expr import Expr, AtomicExpr
@@ -15,6 +15,33 @@ import string
 import re as _re
 import random
 
+class Str(Atom):
+    """
+    Represents string in SymPy.
+
+    Explanation
+    ===========
+
+    Previously, ``Symbol`` was used where string is needed in ``args`` of SymPy
+    objects, e.g. denoting the name of the instance. However, since ``Symbol``
+    represents mathematical scalar, this class should be used instead.
+
+    """
+    __slots__ = ('name',)
+
+    def __new__(cls, name, **kwargs):
+        if not isinstance(name, str):
+            raise TypeError("name should be a string, not %s" % repr(type(name)))
+        obj = Expr.__new__(cls, **kwargs)
+        obj.name = name
+        return obj
+
+    def __getnewargs__(self):
+        return (self.name,)
+
+    def _hashable_content(self):
+        return (self.name,)
+
 
 def _filter_assumptions(kwargs):
     """Split the given dict into assumptions and non-assumptions.
@@ -27,7 +54,6 @@ def _filter_assumptions(kwargs):
     Symbol._sanitize(assumptions)
     return assumptions, nonassumptions
 
-
 def _symbol(s, matching_symbol=None, **assumptions):
     """Return s if s is a Symbol, else if s is a string, return either
     the matching_symbol if the names are the same or else a new symbol
@@ -37,7 +63,7 @@ def _symbol(s, matching_symbol=None, **assumptions):
     Examples
     ========
 
-    >>> from sympy import Symbol, Dummy
+    >>> from sympy import Symbol
     >>> from sympy.core.symbol import _symbol
     >>> _symbol('y')
     y
@@ -94,33 +120,47 @@ def _symbol(s, matching_symbol=None, **assumptions):
     else:
         raise ValueError('symbol must be string for symbol name or Symbol')
 
-
-def _uniquely_named_symbol(xname, exprs=(), compare=str, modify=None, **assumptions):
+def uniquely_named_symbol(xname, exprs=(), compare=str, modify=None, **assumptions):
     """Return a symbol which, when printed, will have a name unique
     from any other already in the expressions given. The name is made
-    unique by prepending underscores (default) but this can be
+    unique by appending numbers (default) but this can be
     customized with the keyword 'modify'.
 
     Parameters
     ==========
 
         xname : a string or a Symbol (when symbol xname <- str(xname))
+
         compare : a single arg function that takes a symbol and returns
             a string to be compared with xname (the default is the str
             function which indicates how the name will look when it
             is printed, e.g. this includes underscores that appear on
             Dummy symbols)
+
         modify : a single arg function that changes its string argument
-            in some way (the default is to prepend underscores)
+            in some way (the default is to append numbers)
 
     Examples
     ========
 
-    >>> from sympy.core.symbol import _uniquely_named_symbol as usym, Dummy
+    >>> from sympy.core.symbol import uniquely_named_symbol
     >>> from sympy.abc import x
-    >>> usym('x', x)
-    _x
+    >>> uniquely_named_symbol('x', x)
+    x0
     """
+    from sympy.core.function import AppliedUndef
+
+    def numbered_string_incr(s, start=0):
+        if not s:
+            return str(start)
+        i = len(s) - 1
+        while i != -1:
+            if not s[i].isdigit():
+                break
+            i -= 1
+        n = str(int(s[i + 1:] or start - 1) + 1)
+        return s[:i + 1] + n
+
     default = None
     if is_sequence(xname):
         xname, default = xname
@@ -129,20 +169,25 @@ def _uniquely_named_symbol(xname, exprs=(), compare=str, modify=None, **assumpti
         return _symbol(x, default, **assumptions)
     if not is_sequence(exprs):
         exprs = [exprs]
-    syms = set().union(*[e.free_symbols for e in exprs])
+    names = set().union((
+        [i.name for e in exprs for i in e.atoms(Symbol)] +
+        [i.func.name for e in exprs for i in e.atoms(AppliedUndef)]))
     if modify is None:
-        modify = lambda s: '_' + s
-    while any(x == compare(s) for s in syms):
+        modify = numbered_string_incr
+    while any(x == compare(s) for s in names):
         x = modify(x)
     return _symbol(x, default, **assumptions)
-
+_uniquely_named_symbol = uniquely_named_symbol
 
 class Symbol(AtomicExpr, Boolean):
     """
     Assumptions:
        commutative = True
 
-    You can override the default assumptions in the constructor:
+    You can override the default assumptions in the constructor.
+
+    Examples
+    ========
 
     >>> from sympy import symbols
     >>> A,B = symbols('A,B', commutative = False)
@@ -273,7 +318,9 @@ class Symbol(AtomicExpr, Boolean):
         return self.class_key(), (1, (self.name,)), S.One.sort_key(), S.One
 
     def as_dummy(self):
-        return Dummy(self.name)
+        # only put commutativity in explicitly if it is False
+        return Dummy(self.name) if self.is_commutative is not False \
+            else Dummy(self.name, commutative=self.is_commutative)
 
     def as_real_imag(self, deep=True, **hints):
         from sympy import im, re
@@ -304,6 +351,9 @@ class Symbol(AtomicExpr, Boolean):
 class Dummy(Symbol):
     """Dummy symbols are each unique, even if they have the same name:
 
+    Examples
+    ========
+
     >>> from sympy import Dummy
     >>> Dummy("x") == Dummy("x")
     False
@@ -376,8 +426,10 @@ class Wild(Symbol):
 
     name : str
         Name of the Wild instance.
+
     exclude : iterable, optional
         Instances in ``exclude`` will not be matched.
+
     properties : iterable of functions, optional
         Functions, each taking an expressions as input
         and returns a ``bool``. All functions in ``properties``
@@ -490,7 +542,7 @@ class Wild(Symbol):
 
 _range = _re.compile('([0-9]*:[0-9]+|[a-zA-Z]?:[a-zA-Z])')
 
-def symbols(names, **args):
+def symbols(names, *, cls=Symbol, **args):
     r"""
     Transform strings into instances of :class:`Symbol` class.
 
@@ -641,7 +693,6 @@ def symbols(names, **args):
         for i in range(len(names) - 1, -1, -1):
             names[i: i + 1] = names[i].split()
 
-        cls = args.pop('cls', Symbol)
         seq = args.pop('seq', as_seq)
 
         for name in names:
@@ -707,6 +758,9 @@ def var(names, **args):
     """
     Create symbols and inject them into the global namespace.
 
+    Explanation
+    ===========
+
     This calls :func:`symbols` with the same arguments and puts the results
     into the *global* namespace. It's recommended not to use :func:`var` in
     library code, where :func:`symbols` has to be used::
@@ -718,17 +772,17 @@ def var(names, **args):
 
     >>> var('x')
     x
-    >>> x
+    >>> x # noqa: F821
     x
 
     >>> var('a,ab,abc')
     (a, ab, abc)
-    >>> abc
+    >>> abc # noqa: F821
     abc
 
     >>> var('x,y', real=True)
     (x, y)
-    >>> x.is_real and y.is_real
+    >>> x.is_real and y.is_real # noqa: F821
     True
 
     See :func:`symbols` documentation for more details on what kinds of
