@@ -1,9 +1,11 @@
 from sympy import (Symbol, zeta, nan, Rational, Float, pi, dirichlet_eta, log,
                    zoo, expand_func, polylog, lerchphi, S, exp, sqrt, I,
-                   exp_polar, polar_lift, O, stieltjes, Abs, Sum)
-from sympy.utilities.randtest import (test_derivative_numerically as td,
+                   exp_polar, polar_lift, O, stieltjes, Abs, Sum, oo)
+from sympy.core.function import ArgumentIndexError
+from sympy.functions.combinatorial.numbers import bernoulli, factorial
+from sympy.testing.pytest import raises
+from sympy.testing.randtest import (test_derivative_numerically as td,
                       random_complex_number as randcplx, verify_numerically as tn)
-from sympy.functions.combinatorial.numbers import bernoulli, factorial, harmonic
 
 x = Symbol('x')
 a = Symbol('a')
@@ -14,17 +16,17 @@ s = Symbol('s')
 
 def test_zeta_eval():
 
-    assert zeta(nan) == nan
-    assert zeta(x, nan) == nan
+    assert zeta(nan) is nan
+    assert zeta(x, nan) is nan
 
     assert zeta(0) == Rational(-1, 2)
-    assert zeta(0, x) == Rational(1, 2) - x
-    assert zeta(0, b) == Rational(1, 2) - b
+    assert zeta(0, x) == S.Half - x
+    assert zeta(0, b) == S.Half - b
 
-    assert zeta(1) == zoo
-    assert zeta(1, 2) == zoo
-    assert zeta(1, -7) == zoo
-    assert zeta(1, x) == zoo
+    assert zeta(1) is zoo
+    assert zeta(1, 2) is zoo
+    assert zeta(1, -7) is zoo
+    assert zeta(1, x) is zoo
 
     assert zeta(2, 1) == pi**2/6
 
@@ -40,24 +42,26 @@ def test_zeta_eval():
     assert zeta(4, -3) == pi**4/90 + Rational(1393, 1296)
     assert zeta(6, -4) == pi**6/945 + Rational(3037465, 2985984)
 
-    assert zeta(-1) == -Rational(1, 12)
+    assert zeta(oo) == 1
+
+    assert zeta(-1) == Rational(-1, 12)
     assert zeta(-2) == 0
     assert zeta(-3) == Rational(1, 120)
     assert zeta(-4) == 0
-    assert zeta(-5) == -Rational(1, 252)
+    assert zeta(-5) == Rational(-1, 252)
 
-    assert zeta(-1, 3) == -Rational(37, 12)
-    assert zeta(-1, 7) == -Rational(253, 12)
+    assert zeta(-1, 3) == Rational(-37, 12)
+    assert zeta(-1, 7) == Rational(-253, 12)
     assert zeta(-1, -4) == Rational(119, 12)
     assert zeta(-1, -9) == Rational(539, 12)
 
     assert zeta(-4, 3) == -17
     assert zeta(-4, -8) == 8772
 
-    assert zeta(0, 1) == -Rational(1, 2)
+    assert zeta(0, 1) == Rational(-1, 2)
     assert zeta(0, -1) == Rational(3, 2)
 
-    assert zeta(0, 2) == -Rational(3, 2)
+    assert zeta(0, 2) == Rational(-3, 2)
     assert zeta(0, -2) == Rational(5, 2)
 
     assert zeta(
@@ -71,7 +75,7 @@ def test_zeta_series():
 
 def test_dirichlet_eta_eval():
 
-    assert dirichlet_eta(0) == Rational(1, 2)
+    assert dirichlet_eta(0) == S.Half
     assert dirichlet_eta(-1) == Rational(1, 4)
     assert dirichlet_eta(1) == log(2)
     assert dirichlet_eta(2) == pi**2/12
@@ -81,6 +85,7 @@ def test_dirichlet_eta_eval():
 def test_rewriting():
     assert dirichlet_eta(x).rewrite(zeta) == (1 - 2**(1 - x))*zeta(x)
     assert zeta(x).rewrite(dirichlet_eta) == dirichlet_eta(x)/(1 - 2**(1 - x))
+    assert zeta(x).rewrite(dirichlet_eta, a=2) == zeta(x)
     assert tn(dirichlet_eta(x), dirichlet_eta(x).rewrite(zeta), x)
     assert tn(zeta(x), zeta(x).rewrite(dirichlet_eta), x)
 
@@ -106,6 +111,10 @@ def test_derivatives():
     assert td(polylog(b, z), z)
     assert td(lerchphi(c, b, x), x)
     assert td(lerchphi(x, b, c), x)
+    raises(ArgumentIndexError, lambda: lerchphi(c, b, x).fdiff(2))
+    raises(ArgumentIndexError, lambda: lerchphi(c, b, x).fdiff(4))
+    raises(ArgumentIndexError, lambda: polylog(b, z).fdiff(1))
+    raises(ArgumentIndexError, lambda: polylog(b, z).fdiff(3))
 
 
 def myexpand(func, target):
@@ -128,7 +137,7 @@ def test_polylog_expansion():
     assert polylog(s, 0) == 0
     assert polylog(s, 1) == zeta(s)
     assert polylog(s, -1) == -dirichlet_eta(s)
-    assert polylog(s, exp_polar(4*I*pi/3)) == polylog(s, exp(4*I*pi/3))
+    assert polylog(s, exp_polar(I*pi*Rational(4, 3))) == polylog(s, exp(I*pi*Rational(4, 3)))
     assert polylog(s, exp_polar(I*pi)/3) == polylog(s, exp(I*pi)/3)
 
     assert myexpand(polylog(1, z), -log(1 - z))
@@ -145,7 +154,7 @@ def test_issue_8404():
 
 
 def test_polylog_values():
-    from sympy.utilities.randtest import verify_numerically as tn
+    from sympy.testing.randtest import verify_numerically as tn
     assert polylog(2, 2) == pi**2/4 - I*pi*log(2)
     assert polylog(2, S.Half) == pi**2/12 - log(2)**2/2
     for z in [S.Half, 2, (sqrt(5)-1)/2, -(sqrt(5)-1)/2, -(sqrt(5)+1)/2, (3-sqrt(5))/2]:
@@ -158,6 +167,9 @@ def test_polylog_values():
             assert tn(polylog(s, z), polylog(s, z, evaluate=False), z,
                 a=2, b=-2, c=5, d=2)
 
+    from sympy import Integral
+    assert polylog(0, Integral(1, (x, 0, 1))) == -S.Half
+
 
 def test_lerchphi_expansion():
     assert myexpand(lerchphi(1, s, a), zeta(s, a))
@@ -167,21 +179,21 @@ def test_lerchphi_expansion():
     assert myexpand(lerchphi(z, -1, a), a/(1 - z) + z/(1 - z)**2)
     assert myexpand(lerchphi(z, -3, a), None)
     # polylog reduction
-    assert myexpand(lerchphi(z, s, S(1)/2),
+    assert myexpand(lerchphi(z, s, S.Half),
                     2**(s - 1)*(polylog(s, sqrt(z))/sqrt(z)
                               - polylog(s, polar_lift(-1)*sqrt(z))/sqrt(z)))
     assert myexpand(lerchphi(z, s, 2), -1/z + polylog(s, z)/z**2)
-    assert myexpand(lerchphi(z, s, S(3)/2), None)
-    assert myexpand(lerchphi(z, s, S(7)/3), None)
-    assert myexpand(lerchphi(z, s, -S(1)/3), None)
-    assert myexpand(lerchphi(z, s, -S(5)/2), None)
+    assert myexpand(lerchphi(z, s, Rational(3, 2)), None)
+    assert myexpand(lerchphi(z, s, Rational(7, 3)), None)
+    assert myexpand(lerchphi(z, s, Rational(-1, 3)), None)
+    assert myexpand(lerchphi(z, s, Rational(-5, 2)), None)
 
     # hurwitz zeta reduction
     assert myexpand(lerchphi(-1, s, a),
                     2**(-s)*zeta(s, a/2) - 2**(-s)*zeta(s, (a + 1)/2))
     assert myexpand(lerchphi(I, s, a), None)
     assert myexpand(lerchphi(-I, s, a), None)
-    assert myexpand(lerchphi(exp(2*I*pi/5), s, a), None)
+    assert myexpand(lerchphi(exp(I*pi*Rational(2, 5)), s, a), None)
 
 
 def test_stieltjes():
@@ -193,12 +205,12 @@ def test_stieltjes():
     assert stieltjes(0, 1) == S.EulerGamma
 
     # Not defined
-    assert stieltjes(nan) == nan
-    assert stieltjes(0, nan) == nan
-    assert stieltjes(-1) == S.ComplexInfinity
-    assert stieltjes(1.5) == S.ComplexInfinity
-    assert stieltjes(z, 0) == S.ComplexInfinity
-    assert stieltjes(z, -1) == S.ComplexInfinity
+    assert stieltjes(nan) is nan
+    assert stieltjes(0, nan) is nan
+    assert stieltjes(-1) is S.ComplexInfinity
+    assert stieltjes(1.5) is S.ComplexInfinity
+    assert stieltjes(z, 0) is S.ComplexInfinity
+    assert stieltjes(z, -1) is S.ComplexInfinity
 
 
 def test_stieltjes_evalf():
@@ -208,8 +220,8 @@ def test_stieltjes_evalf():
 
 
 def test_issue_10475():
-    a = Symbol('a', real=True)
-    b = Symbol('b', positive=True)
+    a = Symbol('a', extended_real=True)
+    b = Symbol('b', extended_positive=True)
     s = Symbol('s', zero=False)
 
     assert zeta(2 + I).is_finite
