@@ -43,16 +43,10 @@ class Curve(GeometrySet):
         When `functions` are specified incorrectly.
         When `limits` are specified incorrectly.
 
-    See Also
-    ========
-
-    sympy.core.function.Function
-    sympy.polys.polyfuncs.interpolate
-
     Examples
     ========
 
-    >>> from sympy import sin, cos, Symbol, interpolate
+    >>> from sympy import sin, cos, interpolate
     >>> from sympy.abc import t, a
     >>> from sympy.geometry import Curve
     >>> C = Curve((sin(t), cos(t)), (t, 0, 2))
@@ -68,6 +62,13 @@ class Curve(GeometrySet):
     Point2D(4, 16)
     >>> C.arbitrary_point(a)
     Point2D(a, a**2)
+
+    See Also
+    ========
+
+    sympy.core.function.Function
+    sympy.polys.polyfuncs.interpolate
+
     """
 
     def __new__(cls, function, limits):
@@ -89,21 +90,21 @@ class Curve(GeometrySet):
             return Point(*[f.subs(old, new) for f in self.functions])
 
     def arbitrary_point(self, parameter='t'):
-        """
-        A parameterized point on the curve.
+        """A parameterized point on the curve.
 
         Parameters
         ==========
 
         parameter : str or Symbol, optional
-            Default value is 't';
-            the Curve's parameter is selected with None or self.parameter
+            Default value is 't'.
+            The Curve's parameter is selected with None or self.parameter
             otherwise the provided symbol is used.
 
         Returns
         =======
 
-        arbitrary_point : Point
+        Point :
+            Returns a point in parametric form.
 
         Raises
         ======
@@ -111,11 +112,6 @@ class Curve(GeometrySet):
         ValueError
             When `parameter` already appears in the functions.
 
-        See Also
-        ========
-
-        sympy.geometry.point.Point
-
         Examples
         ========
 
@@ -132,6 +128,11 @@ class Curve(GeometrySet):
         >>> C.arbitrary_point(Symbol('a'))
         Point2D(2*a, a**2)
 
+        See Also
+        ========
+
+        sympy.geometry.point.Point
+
         """
         if parameter is None:
             return Point(*self.functions)
@@ -146,10 +147,15 @@ class Curve(GeometrySet):
 
     @property
     def free_symbols(self):
-        """
-        Return a set of symbols other than the bound symbols used to
+        """Return a set of symbols other than the bound symbols used to
         parametrically define the Curve.
 
+        Returns
+        =======
+
+        set :
+            Set of all non-parameterized symbols.
+
         Examples
         ========
 
@@ -159,6 +165,7 @@ class Curve(GeometrySet):
         set()
         >>> Curve((t, t**2), (t, a, 2)).free_symbols
         {a}
+
         """
         free = set()
         for a in self.functions + self.limits[1:]:
@@ -168,6 +175,25 @@ class Curve(GeometrySet):
 
     @property
     def ambient_dimension(self):
+        """The dimension of the curve.
+
+        Returns
+        =======
+
+        int :
+            the dimension of curve.
+
+        Examples
+        ========
+
+        >>> from sympy.abc import t
+        >>> from sympy.geometry import Curve
+        >>> C = Curve((t, t**2), (t, 0, 2))
+        >>> C.ambient_dimension
+        2
+
+        """
+
         return len(self.args[0])
 
     @property
@@ -177,12 +203,8 @@ class Curve(GeometrySet):
         Returns
         =======
 
-        functions : list of parameterized coordinate functions.
-
-        See Also
-        ========
-
-        parameter
+        functions :
+            list of parameterized coordinate functions.
 
         Examples
         ========
@@ -193,6 +215,11 @@ class Curve(GeometrySet):
         >>> C.functions
         (t, t**2)
 
+        See Also
+        ========
+
+        parameter
+
         """
         return self.args[0]
 
@@ -206,11 +233,6 @@ class Curve(GeometrySet):
         limits : tuple
             Contains parameter and lower and upper limits.
 
-        See Also
-        ========
-
-        plot_interval
-
         Examples
         ========
 
@@ -220,6 +242,11 @@ class Curve(GeometrySet):
         >>> C.limits
         (t, -2, 2)
 
+        See Also
+        ========
+
+        plot_interval
+
         """
         return self.args[1]
 
@@ -230,12 +257,8 @@ class Curve(GeometrySet):
         Returns
         =======
 
-        parameter : SymPy symbol
-
-        See Also
-        ========
-
-        functions
+        Symbol :
+            returns a bound symbol.
 
         Examples
         ========
@@ -246,6 +269,11 @@ class Curve(GeometrySet):
         >>> C.parameter
         t
 
+        See Also
+        ========
+
+        functions
+
         """
         return self.args[1][0]
 
@@ -257,10 +285,10 @@ class Curve(GeometrySet):
         ========
 
         >>> from sympy.geometry.curve import Curve
-        >>> from sympy import cos, sin
         >>> from sympy.abc import t
         >>> Curve((t, t), (t, 0, 1)).length
         sqrt(2)
+
         """
         integrand = sqrt(sum(diff(func, self.limits[0])**2 for func in self.functions))
         return integrate(integrand, self.limits)
@@ -278,32 +306,48 @@ class Curve(GeometrySet):
         Returns
         =======
 
-        plot_interval : list (plot interval)
-            [parameter, lower_bound, upper_bound]
+        List :
+            the plot interval as below:
+                [parameter, lower_bound, upper_bound]
+
+        Examples
+        ========
+
+        >>> from sympy import Curve, sin
+        >>> from sympy.abc import x, s
+        >>> Curve((x, sin(x)), (x, 1, 2)).plot_interval()
+        [t, 1, 2]
+        >>> Curve((x, sin(x)), (x, 1, 2)).plot_interval(s)
+        [s, 1, 2]
 
         See Also
         ========
 
         limits : Returns limits of the parameter interval
 
-        Examples
-        ========
-
-        >>> from sympy import Curve, sin
-        >>> from sympy.abc import x, t, s
-        >>> Curve((x, sin(x)), (x, 1, 2)).plot_interval()
-        [t, 1, 2]
-        >>> Curve((x, sin(x)), (x, 1, 2)).plot_interval(s)
-        [s, 1, 2]
-
         """
         t = _symbol(parameter, self.parameter, real=True)
         return [t] + list(self.limits[1:])
 
     def rotate(self, angle=0, pt=None):
-        """Rotate ``angle`` radians counterclockwise about Point ``pt``.
+        """This function is used to rotate a curve along given point ``pt`` at given angle(in radian).
 
-        The default pt is the origin, Point(0, 0).
+        Parameters
+        ==========
+
+        angle :
+            the angle at which the curve will be rotated(in radian) in counterclockwise direction.
+            default value of angle is 0.
+
+        pt : Point
+            the point along which the curve will be rotated.
+            If no point given, the curve will be rotated around origin.
+
+        Returns
+        =======
+
+        Curve :
+            returns a curve rotated at given angle along given point.
 
         Examples
         ========
@@ -313,6 +357,7 @@ class Curve(GeometrySet):
         >>> from sympy import pi
         >>> Curve((x, x), (x, 0, 1)).rotate(pi/2)
         Curve((-x, x), (x, 0, 1))
+
         """
         from sympy.matrices import Matrix, rot_axis3
         if pt:
@@ -333,14 +378,20 @@ class Curve(GeometrySet):
     def scale(self, x=1, y=1, pt=None):
         """Override GeometryEntity.scale since Curve is not made up of Points.
 
+        Returns
+        =======
+
+        Curve :
+            returns scaled curve.
+
         Examples
         ========
 
         >>> from sympy.geometry.curve import Curve
-        >>> from sympy import pi
         >>> from sympy.abc import x
         >>> Curve((x, x), (x, 0, 1)).scale(2)
         Curve((2*x, x), (x, 0, 1))
+
         """
         if pt:
             pt = Point(pt, dim=2)
@@ -351,14 +402,20 @@ class Curve(GeometrySet):
     def translate(self, x=0, y=0):
         """Translate the Curve by (x, y).
 
+        Returns
+        =======
+
+        Curve :
+            returns a translated curve.
+
         Examples
         ========
 
         >>> from sympy.geometry.curve import Curve
-        >>> from sympy import pi
         >>> from sympy.abc import x
         >>> Curve((x, x), (x, 0, 1)).translate(1, 2)
         Curve((x + 1, x + 2), (x, 0, 1))
+
         """
         fx, fy = self.functions
         return self.func((fx + x, fy + y), self.limits)
