@@ -276,7 +276,7 @@ def closest_points(*args):
     Examples
     ========
 
-    >>> from sympy.geometry import closest_points, Point2D, Triangle
+    >>> from sympy.geometry import closest_points, Triangle
     >>> Triangle(sss=(3, 4, 5)).args
     (Point2D(0, 0), Point2D(3, 0), Point2D(3, 4))
     >>> closest_points(*_)
@@ -374,7 +374,7 @@ def convex_hull(*args, polygon=True):
     Examples
     ========
 
-    >>> from sympy.geometry import Point, convex_hull
+    >>> from sympy.geometry import convex_hull
     >>> points = [(1, 1), (1, 2), (3, 1), (-5, 2), (15, 4)]
     >>> convex_hull(*points)
     Polygon(Point2D(-5, 2), Point2D(1, 1), Point2D(3, 1), Point2D(15, 4))
@@ -474,7 +474,7 @@ def farthest_points(*args):
     Examples
     ========
 
-    >>> from sympy.geometry import farthest_points, Point2D, Triangle
+    >>> from sympy.geometry import farthest_points, Triangle
     >>> Triangle(sss=(3, 4, 5)).args
     (Point2D(0, 0), Point2D(3, 0), Point2D(3, 4))
     >>> farthest_points(*_)
@@ -604,7 +604,7 @@ def idiff(eq, y, x, n=1):
         dydx = dydx.diff(x)
 
 
-def intersection(*entities, **kwargs):
+def intersection(*entities, pairwise=False, **kwargs):
     """The intersection of a collection of GeometryEntity instances.
 
     Parameters
@@ -668,8 +668,6 @@ def intersection(*entities, **kwargs):
     from .entity import GeometryEntity
     from .point import Point
 
-    pairwise = kwargs.pop('pairwise', False)
-
     if len(entities) <= 1:
         return []
 
