@@ -1,14 +1,13 @@
-# -*- encoding: utf-8 -*-
-from __future__ import print_function, division
+from sympy.core.backend import sympify
+from sympy.physics.vector import Point, ReferenceFrame, Dyadic
+
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 __all__ = ['RigidBody']
 
-from sympy import sympify
-from sympy.physics.vector import Point, ReferenceFrame, Dyadic
-from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 
-class RigidBody(object):
+class RigidBody:
     """An idealized rigid body.
 
     This is essentially a container which holds the various components which
@@ -62,7 +61,8 @@ class RigidBody(object):
     def __str__(self):
         return self._name
 
-    __repr__ = __str__
+    def __repr__(self):
+        return self.__str__()
 
     @property
     def frame(self):
@@ -74,24 +74,6 @@ class RigidBody(object):
             raise TypeError("RigdBody frame must be a ReferenceFrame object.")
         self._frame = F
 
-    def get_frame(self):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.get_frame(self)",
-                useinstead="property sympy.physics.mechanics." +
-                    "RigidBody.frame",
-                deprecated_since_version="1.0", issue=9800).warn()
-        return self.frame
-
-    def set_frame(self, F):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.set_frame(self, F)",
-                useinstead="property sympy.physics.mechanics." +
-                    "RigidBody.frame",
-                deprecated_since_version="1.0", issue=9800).warn()
-        self.frame = F
-
     @property
     def masscenter(self):
         return self._masscenter
@@ -102,24 +84,6 @@ class RigidBody(object):
             raise TypeError("RigidBody center of mass must be a Point object.")
         self._masscenter = p
 
-    def get_masscenter(self):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.get_masscenter(self)",
-                useinstead="property sympy.physics.mechanics." +
-                    "RigidBody.masscenter",
-                deprecated_since_version="1.0", issue=9800).warn()
-        return self.masscenter
-
-    def set_masscenter(self, p):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.set_masscenter(self, p)",
-                useinstead="property sympy.physics.mechanics." +
-                    "RigidBody.masscenter",
-                deprecated_since_version="1.0", issue=9800).warn()
-        self.masscenter = p
-
     @property
     def mass(self):
         return self._mass
@@ -128,22 +92,6 @@ class RigidBody(object):
     def mass(self, m):
         self._mass = sympify(m)
 
-    def get_mass(self):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.get_mass(self)",
-                useinstead="property sympy.physics.mechanics.RigidBody.mass",
-                deprecated_since_version="1.0", issue=9800).warn()
-        return self.mass
-
-    def set_mass(self, m):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.set_mass(self, m)",
-                useinstead="property sympy.physics.mechanics.RigidBody.mass",
-                deprecated_since_version="1.0", issue=9800).warn()
-        self.mass = m
-
     @property
     def inertia(self):
         return (self._inertia, self._inertia_point)
@@ -165,24 +113,6 @@ class RigidBody(object):
                                        self.frame)
         self._central_inertia = I[0] - I_Ss_O
 
-    def get_inertia(self):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.get_inertia(self)",
-                useinstead="property sympy.physics.mechanics." +
-                    "RigidBody.inertia",
-                deprecated_since_version="1.0", issue=9800).warn()
-        return self.inertia
-
-    def set_inertia(self, I):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "RigidBody.set_inertia(self, I)",
-                useinstead="property sympy.physics.mechanics." +
-                    "RigidBody.inertia",
-                deprecated_since_version="1.0", issue=9800).warn()
-        self.inertia = I
-
     @property
     def central_inertia(self):
         """The body's central inertia dyadic."""
@@ -210,6 +140,8 @@ class RigidBody(object):
 
         >>> from sympy.physics.mechanics import Point, ReferenceFrame, outer
         >>> from sympy.physics.mechanics import RigidBody, dynamicsymbols
+        >>> from sympy.physics.vector import init_vprinting
+        >>> init_vprinting(pretty_print=False)
         >>> M, v = dynamicsymbols('M v')
         >>> N = ReferenceFrame('N')
         >>> P = Point('P')
@@ -231,7 +163,7 @@ class RigidBody(object):
         The angular momentum H of a rigid body B about some point O in a frame
         N is given by:
 
-            H = I·w + r×Mv
+            H = I . w + r x Mv
 
         where I is the central inertia dyadic of B, w is the angular velocity
         of body B in the frame, N, r is the position vector from point O to the
@@ -250,6 +182,8 @@ class RigidBody(object):
 
         >>> from sympy.physics.mechanics import Point, ReferenceFrame, outer
         >>> from sympy.physics.mechanics import RigidBody, dynamicsymbols
+        >>> from sympy.physics.vector import init_vprinting
+        >>> init_vprinting(pretty_print=False)
         >>> M, v, r, omega = dynamicsymbols('M v r omega')
         >>> N = ReferenceFrame('N')
         >>> b = ReferenceFrame('b')
@@ -353,7 +287,7 @@ class RigidBody(object):
         Examples
         ========
 
-        >>> from sympy.physics.mechanics import Particle, Point, outer
+        >>> from sympy.physics.mechanics import Point, outer
         >>> from sympy.physics.mechanics import RigidBody, ReferenceFrame
         >>> from sympy import symbols
         >>> b = ReferenceFrame('b')
@@ -374,5 +308,30 @@ class RigidBody(object):
                     "RigidBody.set_potential_energy(self, scalar)",
                 useinstead="property sympy.physics.mechanics." +
                     "RigidBody.potential_energy",
-                deprecated_since_version="1.0", issue=9800).warn()
+                deprecated_since_version="1.5", issue=9800).warn()
         self.potential_energy = scalar
+
+    # XXX: To be consistent with the parallel_axis method in Particle this
+    # should have a frame argument...
+    def parallel_axis(self, point):
+        """Returns the inertia dyadic of the body with respect to another
+        point.
+
+        Parameters
+        ==========
+        point : sympy.physics.vector.Point
+            The point to express the inertia dyadic about.
+
+        Returns
+        =======
+        inertia : sympy.physics.vector.Dyadic
+            The inertia dyadic of the rigid body expressed about the provided
+            point.
+
+        """
+        # circular import issue
+        from sympy.physics.mechanics.functions import inertia
+        a, b, c = self.masscenter.pos_from(point).to_matrix(self.frame)
+        I = self.mass * inertia(self.frame, b**2 + c**2, c**2 + a**2, a**2 +
+                                b**2, -a * b, -b * c, -a * c)
+        return self.central_inertia + I
