@@ -306,7 +306,7 @@ def rep_expectation(expr, **options):
     Examples
     ========
 
-    >>> from sympy.physics.quantum.cartesian import XOp, XKet, PxOp, PxKet
+    >>> from sympy.physics.quantum.cartesian import XOp, PxOp, PxKet
     >>> from sympy.physics.quantum.represent import rep_expectation
     >>> rep_expectation(XOp())
     x_1*DiracDelta(x_1 - x_2)
@@ -411,7 +411,7 @@ def integrate_result(orig_expr, result, **options):
     return result
 
 
-def get_basis(expr, **options):
+def get_basis(expr, *, basis=None, replace_none=True, **options):
     """
     Returns a basis state instance corresponding to the basis specified in
     options=s. If no basis is specified, the function tries to form a default
@@ -462,9 +462,6 @@ def get_basis(expr, **options):
 
     """
 
-    basis = options.pop("basis", None)
-    replace_none = options.pop("replace_none", True)
-
     if basis is None and not replace_none:
         return None
 
