@@ -1,16 +1,14 @@
 """Algorithms for partial fraction decomposition of rational functions. """
 
-from __future__ import print_function, division
-
-from sympy.polys import Poly, RootSum, cancel, factor
-from sympy.polys.polytools import parallel_poly_from_expr
-from sympy.polys.polyoptions import allowed_flags, set_defaults
-from sympy.polys.polyerrors import PolynomialError
 
 from sympy.core import S, Add, sympify, Function, Lambda, Dummy
 from sympy.core.basic import preorder_traversal
+from sympy.polys import Poly, RootSum, cancel, factor
+from sympy.polys.polyerrors import PolynomialError
+from sympy.polys.polyoptions import allowed_flags, set_defaults
+from sympy.polys.polytools import parallel_poly_from_expr
 from sympy.utilities import numbered_symbols, take, xthreaded, public
-from sympy.core.compatibility import range
+
 
 @xthreaded
 @public
@@ -173,7 +171,7 @@ def apart_undetermined_coeffs(P, Q):
         q = q.set_domain(dom)
         F += h*q
 
-    system, result = [], S(0)
+    system, result = [], S.Zero
 
     for (k,), coeff in F.terms():
         system.append(coeff - P.nth(k))
@@ -204,7 +202,7 @@ def apart_full_decomposition(P, Q):
     References
     ==========
 
-    1. [Bronstein93]_
+    .. [1] [Bronstein93]_
 
     """
     return assemble_partfrac_list(apart_list(P/Q, P.gens[0]))
@@ -312,7 +310,7 @@ def apart_list(f, x=None, dummies=None, **options):
     References
     ==========
 
-    1. [Bronstein93]_
+    .. [1] [Bronstein93]_
 
     """
     allowed_flags(options, [])
@@ -367,7 +365,7 @@ def apart_list_full_decomposition(P, Q, dummygen):
     References
     ==========
 
-    1. [Bronstein93]_
+    .. [1] [Bronstein93]_
 
     """
     f, x, U = P/Q, P.gen, []
@@ -429,7 +427,7 @@ def assemble_partfrac_list(partial_list):
     This example is taken from Bronstein's original paper:
 
     >>> from sympy.polys.partfrac import apart_list, assemble_partfrac_list
-    >>> from sympy.abc import x, y
+    >>> from sympy.abc import x
 
     >>> f = 36 / (x**5 - 2*x**4 - 2*x**3 + 4*x**2 + x - 2)
     >>> pfd = apart_list(f)
@@ -468,7 +466,7 @@ def assemble_partfrac_list(partial_list):
     >>> assemble_partfrac_list(pfd)
     -sqrt(2)/(2*(x + sqrt(2))) + sqrt(2)/(2*(x - sqrt(2)))
 
-    See also
+    See Also
     ========
 
     apart, apart_list
@@ -488,10 +486,10 @@ def assemble_partfrac_list(partial_list):
             ad, de = df.variables, df.expr
             # Hack to make dummies equal because Lambda created new Dummies
             de = de.subs(ad[0], an[0])
-            func = Lambda(an, nu/de**ex)
+            func = Lambda(tuple(an), nu/de**ex)
             pfd += RootSum(r, func, auto=False, quadratic=False)
         else:
-            # Assemble in case the roots are given explicitely by a list of algebraic numbers
+            # Assemble in case the roots are given explicitly by a list of algebraic numbers
             for root in r:
                 pfd += nf(root)/df(root)**ex
 
