@@ -151,7 +151,7 @@ def pdsolve(eq, func=None, hint='default', dict=False, solvefun=None, **kwargs):
     ========
 
     >>> from sympy.solvers.pde import pdsolve
-    >>> from sympy import Function, diff, Eq
+    >>> from sympy import Function, Eq
     >>> from sympy.abc import x, y
     >>> f = Function('f')
     >>> u = f(x, y)
@@ -228,7 +228,7 @@ def _handle_Integral(expr, func, order, hint):
         return expr
 
 
-def classify_pde(eq, func=None, dict=False, **kwargs):
+def classify_pde(eq, func=None, dict=False, *, prep=True, **kwargs):
     """
     Returns a tuple of possible pdsolve() classifications for a PDE.
 
@@ -256,7 +256,7 @@ def classify_pde(eq, func=None, dict=False, **kwargs):
     ========
 
     >>> from sympy.solvers.pde import classify_pde
-    >>> from sympy import Function, diff, Eq
+    >>> from sympy import Function, Eq
     >>> from sympy.abc import x, y
     >>> f = Function('f')
     >>> u = f(x, y)
@@ -267,8 +267,6 @@ def classify_pde(eq, func=None, dict=False, **kwargs):
     ('1st_linear_constant_coeff_homogeneous',)
     """
 
-    prep = kwargs.pop('prep', True)
-
     if func and len(func.args) != 2:
         raise NotImplementedError("Right now only partial "
             "differential equations of two variables are supported")
@@ -418,7 +416,7 @@ def checkpdesol(pde, sol, func=None, solve_for_func=True):
     Examples
     ========
 
-    >>> from sympy import Function, symbols, diff
+    >>> from sympy import Function, symbols
     >>> from sympy.solvers.pde import checkpdesol, pdsolve
     >>> x, y = symbols('x y')
     >>> f = Function('f')
@@ -525,10 +523,8 @@ def pde_1st_linear_constant_coeff_homogeneous(eq, func, order, match, solvefun):
     Examples
     ========
 
-    >>> from sympy.solvers.pde import (
-    ... pde_1st_linear_constant_coeff_homogeneous)
     >>> from sympy import pdsolve
-    >>> from sympy import Function, diff, pprint
+    >>> from sympy import Function, pprint
     >>> from sympy.abc import x,y
     >>> f = Function('f')
     >>> pdsolve(f(x,y) + f(x,y).diff(x) + f(x,y).diff(y))
@@ -639,7 +635,7 @@ def pde_1st_linear_constant_coeff(eq, func, order, match, solvefun):
     ========
 
     >>> from sympy.solvers.pde import pdsolve
-    >>> from sympy import Function, diff, pprint, exp
+    >>> from sympy import Function, pprint, exp
     >>> from sympy.abc import x,y
     >>> f = Function('f')
     >>> eq = -2*f(x,y).diff(x) + 4*f(x,y).diff(y) + 5*f(x,y) - exp(x + 3*y)
@@ -701,7 +697,6 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
 
     which can be solved using ``dsolve``.
 
-    >>> from sympy.solvers.pde import pdsolve
     >>> from sympy.abc import x, y
     >>> from sympy import Function, pprint
     >>> a, b, c, G, f= [Function(i) for i in ['a', 'b', 'c', 'G', 'f']]
@@ -719,7 +714,7 @@ def pde_1st_linear_variable_coeff(eq, func, order, match, solvefun):
     ========
 
     >>> from sympy.solvers.pde import pdsolve
-    >>> from sympy import Function, diff, pprint, exp
+    >>> from sympy import Function, pprint
     >>> from sympy.abc import x,y
     >>> f = Function('f')
     >>> eq =  x*(u.diff(x)) - y*(u.diff(y)) + y**2*u - y**2
